
import ByteArrayDataSource;

// Java IO
import java.io.*;
import java.io.IOException;
import java.io.OutputStream;
import java.io.FileInputStream;

// SAX
import org.xml.sax.InputSource;

//JAXP
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.TransformerException;
import javax.xml.transform.Source;
import javax.xml.transform.Result;
import javax.xml.transform.stream.StreamSource;
import javax.xml.transform.sax.SAXResult;


// Apache AXIS
import org.apache.axis.AxisFault;
import org.apache.axis.Message;
import org.apache.axis.MessageContext;
import org.apache.axis.attachments.AttachmentPart;

// Apache FOP
import org.apache.fop.apps.Driver;
import org.apache.fop.apps.FOPException;

// Apache Avalon
import org.apache.avalon.framework.ExceptionUtil;
import org.apache.avalon.framework.logger.Logger;
import org.apache.avalon.framework.logger.ConsoleLogger;

// JAF (Java Activation Framework)
import javax.activation.DataSource;
import javax.activation.URLDataSource;
import javax.activation.DataHandler;

/**
 * @author Administrator
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class FOPWebService {

	/**
	 * Default constructor for FOPService class.  
	 * 
	 */
	public FOPWebService()  throws Exception{
		
	}	
	/**
	 * Exposed as web service. This method needs to be deployed as a
	 * web service onto web/application server. Client can be .Net or Java.
	 * Client sends attachments (e.g. XML/XSLT or FOP) in form of DIME or MIME.  
	 * @param      Output format type (e.g. PDF, PCL, RTF, SVG, XML, AWT, Print, MIF, TXT)
	 * @exception  javax.xml.soap.SOAPException.
	 * 
	 */
	public void formatObjects(String format) throws Exception {
		
        /** AXIS's MesageContext includes Request and Response messages as well 
         *  as attachments.
         */ 
		org.apache.axis.MessageContext messageContext =
		             org.apache.axis.MessageContext.getCurrentContext();

		AttachmentPart attachments[] = (AttachmentPart[]) messageContext.getProperty("Attachments");		

		String[] fileNames = (String[]) messageContext.getProperty("File Names");

	    int cnt = Integer.parseInt((String) messageContext.getProperty("Attachment Count"));
	    
	    
	    if (cnt == 1) {
	    	String foFileName = fileNames[0];
	    	DataHandler dataHandler = renderFO(format,foFileName);
	    	if (dataHandler != null) {
	    		Message responseMessage = messageContext.getResponseMessage();
	    		responseMessage.addAttachmentPart(new AttachmentPart(dataHandler));
	    	}	
	    	
	    } else {
	    	if (cnt == 2) {
	    		String xmlFileName = fileNames[0];
	    		String xslFileName = fileNames[1];
	    		File xmlFile = (new File(xmlFileName)).getAbsoluteFile();
	    		File xslFile = (new File(xslFileName)).getAbsoluteFile();
	    		DataHandler dataHandler = renderXMLXSL(format,xmlFile,xslFile);
    	        if (dataHandler != null) {
    	        	 Message responseMessage = messageContext.getResponseMessage();
                     responseMessage.addAttachmentPart(new AttachmentPart(dataHandler));
	    	    }	
	    		
	    	}	
	    	
	    }
    }	
    
    private DataHandler renderXMLXSL(String format, File xmlFile,File xslFile) throws Exception {
    	ByteArrayOutputStream outputStream = 
                                  new ByteArrayOutputStream();
        Driver driver = new Driver();
        driver = renderFormat(format,driver);
         try {
            //Setup XSLT
            TransformerFactory factory = TransformerFactory.newInstance();
            Transformer transformer = factory.newTransformer(new StreamSource(xslFile));
        
            //Setup input for XSLT transformation
            Source src = new StreamSource(xmlFile);
        
            //Resulting SAX events (the generated FO) must be piped through to FOP
            Result res = new SAXResult(driver.getContentHandler());

            //Start XSLT transformation and FOP processing
            transformer.transform(src, res);
            byte[] content = outputStream.toByteArray();
	        ByteArrayDataSource byteArrayDataSource = new ByteArrayDataSource(outputStream.toByteArray(),"application/pdf");
            DataHandler dataHandler = new DataHandler(byteArrayDataSource);

     	    return dataHandler;
        } finally {
            outputStream.close();
        }
                                  
    	
    }	
    private DataHandler renderFO(String format,String foFileName) throws Exception {
    	
    	ByteArrayOutputStream outputStream = 
                                  new ByteArrayOutputStream();
        InputStream inputStream = new FileInputStream(foFileName);                            
        Driver driver = new Driver(new InputSource(inputStream),outputStream);
        driver = renderFormat(format,driver);
        driver.run();
        byte[] content = outputStream.toByteArray();
	    ByteArrayDataSource byteArrayDataSource = new ByteArrayDataSource(outputStream.toByteArray(),"application/pdf");
        DataHandler dataHandler = new DataHandler(byteArrayDataSource);

    	return dataHandler;
    
    }
	/**
	 * Sets rendering format, based upon client specified format.
	 * @param      Output format type (e.g. PDF, PCL, RTF, SVG, XML, AWT, Print, MIF, TXT)
	 * @param      org.apache.fop.apps.Driver driver
	 * @exception  
	 * 
	 */
    private Driver renderFormat(String format,org.apache.fop.apps.Driver driver) throws Exception   {
    	
    	String mimeType = null;
        try
        {
            //check if output format is supported
            if( format.equalsIgnoreCase("PDF") ||
                format.equalsIgnoreCase("TXT") ||
                format.equalsIgnoreCase("SVG") ||
                format.equalsIgnoreCase("PS")  ||
                format.equalsIgnoreCase("PCL")) {
                	if (format.equalsIgnoreCase("PDF")) {
                	   driver.setRenderer(Driver.RENDER_PDF);
                	   mimeType = "application/pdf";
                	
                    }
                	
                	if (format.equalsIgnoreCase("TXT")) {
                	   driver.setRenderer(Driver.RENDER_TXT);
                	   mimeType = "application/txt";
                	
                    }
                	if (format.equalsIgnoreCase("SVG")) {
                	   driver.setRenderer(Driver.RENDER_SVG);
                	   mimeType = "application/svg";
                	
                    }
                	if (format.equalsIgnoreCase("PS")) {
                	   driver.setRenderer(Driver.RENDER_SVG);
                 	   mimeType = "application/ps";
               	
                    }
                	if (format.equalsIgnoreCase("PCL")) {
                	   driver.setRenderer(Driver.RENDER_PCL);
                	   mimeType = "application/pcl";
                	
                    }
     
             }
        } catch (Exception fe) {
        	
        }
        return driver;     
   }

}
