using System;
using System.Xml.XPath;
using System.Diagnostics;
using System.Data;
using System.Web;
using System.Net;
using System.Web.Services.Protocols;
using System.ComponentModel;
using Microsoft.Web;
using System.Web.Services;
using Microsoft.Web.Services;
using System.Xml;
using System.IO;
using System.Text;
using System.Windows.Forms;

namespace CustomFilterLibrary
{
	/// <summary>
	/// Using WSE Custom filter, log SOAp fault messages to Windows Event Log.
	/// </summary>
	public class SoapFaultInputFilter : SoapInputFilter
	{
		private string _EventLogSource = "FOPWebService";		
		private string _MachineName = ".";
		private string _EventLogName = "SoapFaultInputFilter";
		

		public SoapFaultInputFilter()
		{

			//
			// TODO: Add constructor logic here
			//
			//			Console.WriteLine("SOAPFaultOutputFilter constructor..");
		}

		// getter and setter methods for properties
		public string EventLogName 
		{
			get 
			{
				return this._EventLogName; 
			}
			set 
			{ 
				this._EventLogName = value;
			}
		}
		public string MachineName 
		{
			get 
			{
				return this._MachineName; 
			}
			set 
			{ 
				this._MachineName = value;
			}
		}
		public string EventLogSource 
		{
			get 
			{
				return this._EventLogSource; 
			}
			set 
			{ 
				this._EventLogSource = value;
			}
		}
		public override void ProcessMessage(SoapEnvelope envelope) 

		{

			XmlNamespaceManager nsMgr =
				new XmlNamespaceManager(envelope.NameTable);

			
			nsMgr.AddNamespace("soapenv",
				"http://schemas.xmlsoap.org/soap/envelope/");
			XmlElement fault = (XmlElement)
				envelope.Body.SelectSingleNode("soapenv:Fault", nsMgr);
			
//			String soapfault = envelope.Context["soapfault"] as string;

//			WriteToEventLog(soapfault, "WSE");

			// if there is a fault...
			if (fault != null)
			{
				// get fault children
				XmlNode faultcode = fault.SelectSingleNode("faultcode");
				XmlNode faultstring = fault.SelectSingleNode("faultstring");
				XmlNode faultactor = fault.SelectSingleNode("faultactor");
				XmlNode detail = fault.SelectSingleNode("detail");

				// format fault children into string
				StringBuilder faultcodeBuilder = new StringBuilder("fault code = " +
					                                               ((faultcode != null) ? faultcode.InnerText : "")); 
					                                  
				StringBuilder faultstringBuilder = new StringBuilder("fault string = " +
                                                  					((faultstring != null) ? faultstring.InnerText : "")); 
				StringBuilder faultactorBuilder = new StringBuilder("fault actor = " +
																	((faultactor != null) ? faultactor.InnerText : "")); 
				StringBuilder faultdetailBuilder = new StringBuilder("fault detail = " +
																	((detail != null) ? detail.InnerText : "")); 
				
				string output = faultcodeBuilder.ToString() + "\n" +
                                faultstringBuilder.ToString() + "\n" +
                                faultactorBuilder.ToString() + "\n" +
                                faultdetailBuilder.ToString();
				MessageBox.Show(output,"SOAP fault - Input Filter",
					            MessageBoxButtons.OK,MessageBoxIcon.Error);
                                        
				string msg = string.Format("{0}\n{1}\n{2}\n\n{3}",
					((faultcode != null) ? faultcode.InnerText : ""),
					((faultstring != null) ? faultstring.InnerText : ""),
					((faultactor != null) ? faultactor.InnerText : ""),
					((detail != null) ? detail.OuterXml : ""));

//				Console.WriteLine("Message = " + msg);                                                                        
				// write raw fault body to a byte array
				MemoryStream stm = new MemoryStream();
				StreamWriter sw = new StreamWriter(stm);
				sw.Write(fault.OuterXml);
				sw.Flush();
				byte[] buf = stm.ToArray();
      
				// write fault data to event log  
				WriteToEventLog(output, "WSE", buf);

				sw.Close();
			}

		}


		private void WriteToEventLog(string msg,string src, byte[] buf)
		{
			EventLog eventLog = new EventLog();
 			try
			{
				// write fault to event log, creating if necessary
				eventLog.Source = this._EventLogSource;
				eventLog.MachineName = this._MachineName;
				eventLog.Log = this._EventLogName;
				eventLog.WriteEntry(msg, EventLogEntryType.Error, 0, 0, buf);
			}
			finally
			{
				if (eventLog != null) eventLog.Dispose();
			}
		}
	}
}
